package de.uni_marburg.powersort.sort;

import de.uni_marburg.powersort.FinnSort.FinnSort;
import de.uni_marburg.powersort.FinnSort.FasterFinnSort;
import de.uni_marburg.powersort.MSort.IMPL_M_1;
import de.uni_marburg.powersort.MSort.IMPL_M_2;
import de.uni_marburg.powersort.benchmark.NaturalOrder;
import de.uni_marburg.powersort.sort.dpqs.DualPivotQuicksort;

public enum SortEnum {
    TIM_SORT,
    FASTER_FINN_SORT,
    ASORT,
    FINN_SORT,
    IMPL_M_10,
    IMPL_M_20,
    DPQS,
    QUICK_SORT,
    MERGE_SORT,
    BUBBLE_SORT;

    public SortImpl getSortImpl() {
        return switch (this) {
            case BUBBLE_SORT -> array -> BubbleSort.sort(array, NaturalOrder.INSTANCE);
            case QUICK_SORT -> array -> QuickSort.sort(array, NaturalOrder.INSTANCE);
            case DPQS -> array -> DualPivotQuicksort.sort(array, 0, array.length);
            case MERGE_SORT -> array -> MergeSort.legacyMergeSort(array, NaturalOrder.INSTANCE);
            case TIM_SORT -> array -> TimSort.sort(array, 0, array.length, NaturalOrder.INSTANCE, null, 0, 0);
            case FINN_SORT -> array -> FinnSort.sort(array, NaturalOrder.INSTANCE);
            case IMPL_M_10 -> array -> IMPL_M_1.powerSort(array,NaturalOrder.INSTANCE);
            case IMPL_M_20 -> array -> IMPL_M_2.sort(array, 0, array.length, NaturalOrder.INSTANCE, null, 0, 0);
            case FASTER_FINN_SORT -> array -> FasterFinnSort.sort(array, 0, array.length, NaturalOrder.INSTANCE, null, 0, 0);
            case ASORT -> array -> ASort.sort(array, NaturalOrder.INSTANCE);
        };
    }
}
