package de.uni_marburg.powersort.FinnSort;

import java.util.Comparator;

import static de.uni_marburg.powersort.FinnSort.FasterFinnSort.MIN_MERGE;
import static de.uni_marburg.powersort.FinnSort.FasterFinnSort.binarySort;
import static de.uni_marburg.powersort.FinnSort.FasterFinnSort.countRunAndMakeAscending;

public class FasterFinnSortWrapper {
    /**
     * Copy of `FasterFinnSort.java`. Param `ts` has been added to allow dependency injection for testing.
     *
     * @param fs: If non-null, the creation of `ts` with `new FasterFinnSort<>(a, c, work, workBase, workLen)` in this methods body is skipped. Instead, the given `ts` is used.
     */
    public static <T> void sort(T[] a, int lo, int hi, Comparator<? super T> c,
                                T[] work, int workBase, int workLen,
                                FasterFinnSort<T> fs, boolean forcePowersortShortArray) {
        assert c != null && a != null && lo >= 0 && lo <= hi && hi <= a.length;
        int nRemaining = hi - lo;
        if (nRemaining < 2)
            return;  // Arrays of size 0 and 1 are always sorted

        // If array is small, do a "mini-TimSort" with no merges

        if (!forcePowersortShortArray && nRemaining < MIN_MERGE) {
            int initRunLen = countRunAndMakeAscending(a, lo, hi, c);
            binarySort(a, lo, hi, lo + initRunLen, c);
            return;
        }

        /**
         * March over the array once, left to right, finding natural runs,
         * extending short natural runs to minRun elements, and merging runs
         * to maintain stack invariant.
         */
        if (fs == null) fs = new FasterFinnSort<>(a, c, work, workBase, workLen, hi - lo);
        int minRun = fs.minRunLength(nRemaining);
        do {
            // Identify next run
            int runLen = countRunAndMakeAscending(a, lo, hi, c);

            // If run is short, extend to min(minRun, nRemaining)
            if (runLen < minRun) {
                int force = nRemaining <= minRun ? nRemaining : minRun;
                binarySort(a, lo, lo + force, lo + runLen, c);
                runLen = force;
            }

            // Push run onto pending-run stack, and maybe merge
            fs.pushRun(lo, runLen);
            fs.mergeCollapse();

            // Advance to find next run
            lo += runLen;
            nRemaining -= runLen;
        } while (nRemaining != 0);

        // Merge all remaining runs to complete sort
        assert lo == hi;
        fs.mergeForceCollapse();
        assert fs.stackSize == 1;
    }
}
